@@ -8,69 +8,75 @@ BEGIN{
 #XML::PARSER# use XML::Parser;
 
  use Data::Dumper;
+ use LWP::Simple;
  use Exporter ();
  use vars qw($c %v $q @dtcontext %dtcontextcount @dtatributes );
  eval "use bytes";
  if (my $m = $INC{"bytes.pm"}) {require bytes; import bytes;}
  @ISA=qw(Exporter);
- @EXPORT=qw(dt dtstring inctxt ctxt mkdtskel mkdtdskel toxml MMAPON $c %v $q 
-         @dtcontext %dtcontextcount @dtatributes pathdt pathdtstring );
- $VERSION = '0.23';
+ @EXPORT=qw(&dt &dtstring &dturl &inctxt &ctxt &mkdtskel &mkdtdskel &toxml 
+         &MMAPON $c %v $q &xmltree &pathdturl
+         @dtcontext %dtcontextcount @dtatributes &pathdt &pathdtstring );
+ $VERSION = '0.24';
 }
 
 =head1 NAME
 
-XML::DT - a package for down translation of XML to strings
+XML::DT - a package for down translation of XML files
 
 =head1 SYNOPSIS
 
     use XML::DT;
 
     %xml=( 'music'    => sub{"Music from: $c\n"},
-           'lyrics'   => sub{"Lyrics from:$c\n (the value of attribute
-                               IN is:$v{IN}\n)"},
+           'lyrics'   => sub{"Lyrics from: $v{name}\n"},
            'title'    => sub{ uc($c) },
-           '-default' => sub{"$q:$c"},
-           '-outputenc' => 'ISO-8859-1');
-    
+           '-default' => sub{"$q:$c"} );
+
     print dt($filename,%xml);
 
-    print dtstring("<arq>
-                    <title>Vejam Bem</title>
-                    <music>Zeca Afonso</music>
-                    </arq>",%xml);
-
-    inctxt('music/lyrics')
-    inctxt('music.*')
-
-    ctxt(1)       /* the father element */
-
-    mkdtskel($file)
-    mkdtskel("-latin1", $file)
-    mkdtdskel($file)
-
 =head1 DESCRIPTION
 
-This module processes XML files with an approach similar to OMNIMARK. As 
-XML parser it uses XML::Parser or XML::LibXML module in an indenpendent way. At
-configure stage, you should choose one of the backends.
+This module processes XML files with an approach similar to
+OMNIMARK. As XML parser it uses XML::Parser or XML::LibXML module in
+an independent way. At configure stage, you should choose one of the
+back-ends.
 
-Down translation function C<dt> receives a filename and a set of expressions
-(functions) defining the processing and associated values for each element.
+If you use XML::LibXML module as backend, you can parse HTML files as
+if they were XML files. For this, you must supply an extra option to
+the hash:
 
-C<dtstring> is similar but takes input from a string instead of a file.
+  %hander = ( -html => 1,
+              ...
+            );
+
+=head2 C<dt> function
+
+Down translation function C<dt> receives a filename and a set of
+expressions (functions) defining the processing and associated values
+for each element.
+
+=head2 C<dtstring> function
+
+C<dtstring> works in a similar way with C<dt> but takes input from a
+string instead of a file.
+
+=head2 C<dturl> function
+
+C<dturl> works in a similar way with C<dt> but takes input from an
+Internet url instead of a file.
 
 =head2 C<pathdt> function
 
-The C<pathdt> function uses a subset of XPath as key in the handler. Example:
+The C<pathdt> function is a C<dt> function which can handle a subset
+of XPath on handler keys. Example:
 
-   %handler = (
+  %handler = (
         "article/title" => sub{ toxml("h1",{},$c) },
         "section/title" => sub{ toxml("h2",{},$c) },
         "title"         => sub{ $c },
         "//image[@type='jpg']" => sub{ "JPEG: <img src=\"$c\">" },
         "//image[@type='bmp']" => sub{ "BMP: sorry, no bitmaps on the web" },
-        ...
   )
 
   pathdt($filename,%handler);
@@ -90,23 +96,64 @@ Here are some examples of valid XPath expressions under XML::DT:
   //*[name()='bbb']               - complex way of saying "//bbb"
   //*[starts-with(name(),'aa')]   - an element named "aa.*"
   //*[contains(name(),'c')]       - an element       ".*c.*"
-  //aaa[string-length(name())=4]  -                  "...."
+  //aaa[string-length(name())=4]                     "...."
   //aaa[string-length(name())&lt;4]                  ".{1,4}"
   //aaa[string-length(name())&gt;5]                  ".{5,}"
 
-For more information, visit www.w3c.org or try a tutorial under www.zvon.org
+Note that not all XPath is currently handled by XML::DT. A lot of
+XPath will never be added to XML::DT because is not in accordance with
+the down translation model. For more documentation about XPath check
+the specification at http://www.w3c.org or some tutorials under
+http://www.zvon.org
+
+=head2 C<pathdtstring> function
+
+Like the C<dtstring> function but supporting XPath.
+
+=head2 C<pathdturl> function
+
+Like the C<dturl> function but supporting XPath.
 
 =head2 C<inctxt> function
 
-C<inctxt(pattern)> is true if the actual element path matches the provided 
-pattern. This function is meant to be used in the element functions in order
-to achieve context dependent processing. 
+C<inctxt(pattern)> is true if the actual element path matches the
+provided pattern. This function is meant to be used in the element
+functions in order to achieve context dependent processing.
 
-=head2 User provided element processing functions
+=head2 C<toxml> function
 
-The user must provide an HASH with a function for each element,
-that computes element output. Functions can use the element name C<$q>, 
-the element content C<$c> and the attribute values hash C<%v>. 
+This is the default "-default" function. It can be used to generate
+XML based on C<$c> C<$q> and C<%v> variables. Example: add a new
+attribute to element C<ele1> without changing it:
+
+  %handler=( ...
+             ele1 => sub { $v{at1} = "v1"; toxml(); },
+           )
+
+C<toxml> can also be used with 3 arguments: tag, attributes and contents
+
+  toxml("a",{href=> "http://local/f.html"}, "example")
+
+returns:
+
+  <a href='http://local/f.html'>example</a>
+
+=head2 C<xmltree> function
+
+This simple function just makes a HASH reference:
+
+  { -c => $c, -q => $q, all_the_other_attributes }
+
+The function C<toxml> understands this structure and makes XML with it.
+
+
+
+
+=head1 User provided element processing functions
+
+The user must provide an HASH with a function for each element, that
+computes element output. Functions can use the element name C<$q>, the
+element content C<$c> and the attribute values hash C<%v>.
 
 All those global variables are defined in C<$CALLER::>.
 
@@ -117,9 +164,9 @@ interior elements return values.
 
 =head2 C<-default> function
 
-When a element has no associated function, the function associated with 
-C<-default> called. If no C<-default> function is defined the default function 
-returns a XML like string for the element.
+When a element has no associated function, the function associated
+with C<-default> called. If no C<-default> function is defined the
+default function returns a XML like string for the element.
 
 When you use C</-type> definitions, you often need do set C<-default>
 function to return just the contents: C<sub{$id}>.
@@ -137,8 +184,9 @@ define the input encoding in the XML file:
 
 =head2 C<-pcdata> function
 
-C<-pcdata> function is used to define transformation over the contents.
-Typically this function should look at context (see C<inctxt> function)
+C<-pcdata> function is used to define transformation over the
+contents.  Typically this function should look at context (see
+C<inctxt> function)
 
 The default C<-pcdata> function is the identity
 
@@ -150,61 +198,79 @@ Example of use: initialization of side-effect variables
 
 =head2 C<-end> function
 
-Function to be executed after processing XML file.
-I can use C<$c> content value.
-The value returned by C<-end> will be the C<dt> return value.
+Function to be executed after processing XML file.  I can use C<$c>
+content value.  The value returned by C<-end> will be the C<dt> return
+value.
 
-Example of use: post-processing of returned contents 
+Example of use: post-processing of returned contents
 
-=head2 C<toxml> function
 
-This is the default "-default" function. It can be used to generate xml
-based on C<$c> C<$q> and C<%v> variables. Example: add a new attribute to
-element C<ele1> without changing it:
-
-  %handler=( ...
-             ele1 => sub { $v{at1} = "v1"; toxml(); },
-           )
-
-C<toxml> can also be used with 3 arguments: tag, attrigutes and contents
-
-  toxml("a",{href=> "http://local/f.html"}, "example")
-
-returns:
-
-  <a href='http://local/f.html'>example</a>
 
 =head1 Elements with values other than strings (C<-type>)
 
 By default all elements return strings, and contents (C<$c>) is the
 concatenation of the strings returned by the sub-elements.
 
-In some situations the XML text contains values that are better processed as
-a structured type.
+In some situations the XML text contains values that are better
+processed as a structured type.
 
 The following types (functors) are available:
 
-     STR  -> concatenates all the subelements returned values (DEFAULT)
-          all the subelement should return strings to be concatenated
-     SEQ  -> makes an ARRAY with all the sub elements contents; attributes are
-          ignored (they should be processed in the subelement). (returns a ref)
-     SEQH -> makes an ARRAY of HASH with all the sub elements (returns a ref);
-          for each subelement: 
-                 -q  => element name
-                 -c  => contents
-                 at1 => at value1    for each attribute
-     MAP  -> makes an HASH with the sub elements; keys are the sub-element
-          names, values are their contents. Attributes are ignored. (they should
-          be processed in the subelement) (returns a ref)
-     MULTIMAP -> makes an HASH of ARRAY; keys are the sub-element names;
-         values are lists of contents; attributes are ignored (they should be
-         processed in the subelement); (returns a ref)
-     MMAPON(elementlist) -> makes an HASH with the subelements; 
-          keys are the sub-element names, values are their contents; 
-          attributes are ignored (they should be processed in the subelement);
-          for all the elements contained in the elementlist, it is created 
-          an ARRAY with their contents. (returns a ref)
-     ZERO -> don't process the subelements; return ""
+=over 4
+
+=item STR
+
+concatenates all the sub-elements returned values (DEFAULT) all the
+sub-element should return strings to be concatenated;
+
+=item SEQ
+
+makes an ARRAY with all the sub elements contents; attributes are
+ignored (they should be processed in the sub-element). (returns a ref)
+If you have different types of sub-elements, you should use SEQH
+
+=item SEQH
+
+makes an ARRAY of HASH with all the sub elements (returns a ref); for
+each sub-element:
+
+  -q  => element name
+  -c  => contents
+  at1 => at value1    for each attribute
+
+=item MAP
+
+makes an HASH with the sub elements; keys are the sub-element names,
+values are their contents. Attributes are ignored. (they should be
+processed in the sub-element) (returns a ref)
+
+=item MULTIMAP
+
+makes an HASH of ARRAY; keys are the sub-element names; values are
+lists of contents; attributes are ignored (they should be processed in
+the sub-element); (returns a ref)
+
+=item MMAPON(element-list)
+
+makes an HASH with the sub-elements; keys are the sub-element names,
+values are their contents; attributes are ignored (they should be
+processed in the sub-element); for all the elements contained in the
+element-list, it is created an ARRAY with their contents. (returns a
+ref)
+
+=item XML
+
+return a reference to an HASH with:
+
+  -q  => element name
+  -c  => contents
+  at1 => at value1    for each attribute
+
+=item ZERO
+
+don't process the sub-elements; return ""
+
+=back
 
 When you use C</-type> definitions, you often need do set C<-default>
 function returning just the contents C<sub{$id}>.
@@ -227,7 +293,7 @@ with the following f.xml
       <id>U.M.</id>
       <name>University of Minho</name>
       <tels>
-        <item>1111</item> 
+        <item>1111</item>
         <item>1112</item>
         <item>1113</item>
       </tels>
@@ -275,9 +341,9 @@ Example:
 
   perl -MXML::DT -e 'mkdtdskel "f.xml"' > f.dtd
 
-=head1 BUGS
+=head1 SEE ALSO
 
-This section is out of date...
+mkdtskel(1) and mkdtdskel(1)
 
 =head1 AUTHORS
 
@@ -285,13 +351,13 @@ Home for XML::DT;
 
   http://natura.di.uminho.pt/~jj/perl/XML/
 
-Jose Joao, <jj@di.uminho.pt>
+Jose Joao Almeida, <jj@di.uminho.pt>
 
 Alberto Manuel Simes, <albie@alfarrabio.di.uminho.pt>
 
 thanks to
 
-  Michel Rodriguez <mrodrigu@ieee.org>
+  Michel Rodriguez    <mrodrigu@ieee.org>
   Jos Carlos Ramalho <jcr@di.uminho.pt>
   Mark A. Hillebrand
 
@@ -318,7 +384,12 @@ sub dt {
 #XML::LIBXML#  $parser->expand_entities(1) if defined $xml{'-noexpand'} && $xml{'-noexpand'};
 
 #XML::LIBXML## parse the file
-#XML::LIBXML#  my $doc = $parser->parse_file($file);
+#XML::LIBXML#  my $doc;
+#XML::LIBXML#  $doc = $parser->parse_file($file)      unless( $xml{'-html'});
+#XML::LIBXML#  if    ( $xml{'-html'}) {
+#XML::LIBXML#    eval{$doc = $parser->parse_html_file($file);};
+#XML::LIBXML#    if ($@) { return undef; }
+#XML::LIBXML#  }
 
 #XML::LIBXML## get the document root element
 #XML::LIBXML#  my $tree = $doc->getDocumentElement();
@@ -368,6 +439,22 @@ sub pathdtstring{
   return dtstring($string,%h);
 }
 
+sub pathdturl{
+  my $url = shift;
+  my %h = pathtodt(@_);
+  return dturl($url,%h);
+}
+
+sub dturl{
+  my $url = shift;
+  my $contents = get($url);
+  if ($contents) {
+    return dtstring($contents, @_);
+  } else {
+    return undef;
+  }
+}
+
 sub dtstring
 { my ($string,%xml)=@_;
 
@@ -388,7 +475,15 @@ sub dtstring
 #XML::LIBXML#  $parser->expand_entities(0) if defined $xml{'-noexpand'} && $xml{'-noexpand'};
 
 #XML::LIBXML## parse the string
-#XML::LIBXML#  my $doc = $parser->parse_string($string);
+#XML::LIBXML#  my $doc;
+
+#XML::LIBXML#  $doc = $parser->parse_string($string)      unless( $xml{'-html'});
+#XML::LIBXML#  if    ( $xml{'-html'}) {
+#XML::LIBXML#    eval{$doc = $parser->parse_html_string($string);};
+#XML::LIBXML#    if ($@) { return undef; }
+#XML::LIBXML#  }
+
+
 
 #XML::LIBXML## get the document root element
 #XML::LIBXML#  my $tree = $doc->getDocumentElement();
@@ -425,7 +520,7 @@ sub pathdt{
   return dt($file,%h);
 }
 
-# Testa os predicados do XPath
+# Parsing dos predicados do XPath
 sub testAttr {
   my $atr = shift;
   for ($atr) {
@@ -571,7 +666,20 @@ sub omni{
 
   while(@l) {
 #XML::LIBXML#    my $tree = shift @l;
-#XML::LIBXML#    $name = $tree->getName();
+#XML::LIBXML#    if (ref($tree) eq "XML::LibXML::CDATASection") {
+#XML::LIBXML#      $name = "-pcdata";
+#XML::LIBXML#      $val = $tree->getData();
+#XML::LIBXML#      $aux= (defined($xml->{-outputenc}) && $xml->{-outputenc} eq 'ISO-8859-1')
+#XML::LIBXML#	            ?lat1::utf8($val): $val ;
+#XML::LIBXML#      if (defined $xml->{-pcdata}) {
+#XML::LIBXML#	      push(@dtcontext,"-pcdata");
+#XML::LIBXML#	      $c = $aux;
+#XML::LIBXML#	      $aux = &{$xml->{-pcdata}};
+#XML::LIBXML#	      pop(@dtcontext);
+#XML::LIBXML#      }
+#XML::LIBXML#    } else {
+#XML::LIBXML#      $name = $tree->getName();
+#XML::LIBXML#    }
 #XML::PARSER#    ($name,$val,@l) = @l;
 
 #XML::LIBXML#    if (ref($tree) eq "XML::LibXML::Comment") {
@@ -614,7 +722,7 @@ sub omni{
       shift(@dtatributes);
       pop(@dtcontext); $dtcontextcount{$name}--;
     }
-    if   ($type eq "STR"){ $r .= $aux ;}
+    if   ($type eq "STR"){ if ($aux) {$r .= $aux} ;}
     elsif($type eq "SEQ" or $type eq "ARRAY"){
       push(@$r, $aux) unless whitepc($aux, $name);}
     elsif($type eq "SEQH" or $type eq "ARRAYHASH"){
@@ -626,16 +734,16 @@ sub omni{
       if(not whitepc($aux,$name)){
 	if(! $typeargs{$name}) {
 	  warn "duplicated tag $name\n" if(defined($r->{$name}));
-	  $r->{$name}=$aux }
+	  $r->{$name} = $aux }
 	else { push(@{$r->{$name}},$aux) unless whitepc($aux,$name)}}
     }
     elsif($type eq "MAP" or $type eq "HASH"){
       if(not whitepc($aux,$name)){
 	warn "duplicated tag $name\n" if(defined($r->{$name}));
-	$r->{$name}=$aux }}
+	$r->{$name} = $aux }}
     elsif($type eq "MULTIMAP"){
       push(@{$r->{$name}},$aux) unless whitepc($aux,$name)}
-    elsif($type eq "NONE"){ $r=$aux;}
+    elsif($type eq "NONE"){ $r = $aux;}
     else { $r="undefined type !!!"}
   }
   $r;
@@ -647,55 +755,103 @@ sub omniele {
   my $aux;
   ($q, $c, $aux) = @_;
 
-  %v=%$aux;
+  %v = %$aux;
 
   if (defined($xml->{-outputenc}) && $xml->{-outputenc} eq 'ISO-8859-1'){
     for (keys %v){ $v{$_} = lat1::utf8($v{$_}) ; }
   }
 
-  if   (defined $xml->{$q}) {&{$xml->{$q}} }
-  elsif(defined $xml->{'-default'}) {&{$xml->{'-default'} }}
-  else {toxml();}
+  if    (defined $xml->{$q}) {&{$xml->{$q}} }
+  elsif (defined $xml->{'-default'}) {&{$xml->{'-default'} }}
+  else  {toxml();}
+}
+
+sub xmltree {
+  +{'-c' => $c, '-q' => $q, %v}
 }
 
 sub toxmlp {
-  my($q,$v,$c ) = @_;
-  if($q eq "-pcdata") { $c}
-  else {"<$q".  join("",map {" $_=\"$v->{$_}\""} keys %$v ) . ">$c</$q>" }
+  my ($q,$v,$c ) = @_;
+  if ($q eq "-pcdata") { $c }
+  else {"<$q" . join("",map {" $_=\"$v->{$_}\""} keys %$v ) . ">$c</$q>" }
 }
 
 sub toxml {
-     if(@_ == 3){return toxmlp(@_)}
-     return "" if (defined $ty{$q} && $ty{$q} eq "ZERO");
-     if(not ref($c)){  toxmlp($q,\%v,$c)}
-     elsif (ref($c) eq "ARRAY") { 
-       if($ty{$q} eq "SEQH") {
-          toxmlp($q,{},
+  my ($q,$v,$c);
+
+  if (not @_) {
+    ($q,$v,$c) = ($XML::DT::q, \%XML::DT::v, $XML::DT::c);
+  } elsif (ref($_[0])) {
+    $c = shift;
+  } else {
+    ($q,$v,$c) = @_;
+  }
+
+  if (not ref($c)) {
+    if ($q eq "-pcdata") {
+      return $c
+    }
+    openTag($q,$v) . "$c</$q>"
+  }
+  elsif (ref($c) eq "HASH" && $c->{'-q'} && $c->{'-c'}) {
+    my %a = %$c;
+    my ($q,$c) = delete @a{"-q","-c"};
+    toxml($q,\%a,$c);
+  } elsif (ref($c) eq "HASH") {
+    openTag($q,$v).
+      join("",map {($_ ne "-pcdata")
+		     ? ( (ref($c->{$_}) eq "ARRAY")
+                         ? "<$_>".
+			 join("</$_>\n<$_>", @{$c->{$_}}).
+			 "</$_>\n" 
+                         : toxml($_,{},$c->{$_})."\n" )
+		       : () }
+	   keys %{$c} ) .
+	     "$c->{-pcdata}</$q>" } ########  "NOTYetREady"
+  elsif (ref($c) eq "ARRAY") {
+    if($ty{$q} eq "SEQH"){toxml($q,$v,join("\n",map {toxml($_)} @$c))}
+    else                 {toxml($q,\%v, join("",@{$c}))}
+  }
+}
+
+sub openTag{
+  "<$_[0]". join("",map {" $_=\"$_[1]{$_}\""} keys %{$_[1]} ).">"
+}
+
+sub toxml1 {
+  if(@_ == 3){return toxmlp(@_)}
+  return "" if (defined $ty{$q} && $ty{$q} eq "ZERO");
+  if(not ref($c)){  toxmlp($q,\%v,$c)}
+  elsif (ref($c) eq "ARRAY") {
+    if($ty{$q} eq "SEQH") {
+      toxmlp($q,{},
              join("",map {my %a=%$_; 
                           delete @a{"-q","-c"}; 
-                          toxmlp($_->{-q},\%a,$_->{-c}) } @{$c} )) 
-       }
-       else { toxmlp($q,\%v, join("",@{$c}))}
-     }
-     elsif (ref($c) eq "HASH") {   "<$q".
-        join("",map {" $_=\"$v{$_}\""} keys %v ) . ">" .
-        join("",map {($_ ne "-pcdata") 
-                     ? ( (ref($c->{$_}) eq "ARRAY")
-                         ? "<$_>". 
-                           join("</$_>\n<$_>", @{$c->{$_}}).
-                           "</$_>\n" 
-                         : "<$_>$c->{$_}</$_>\n" )
-                     : () } 
-                    keys %{$c} ) .
-        "$c->{-pcdata}</$q>" }
+                          toxmlp($_->{'-q'},\%a,$_->{-c}) } @{$c} ))
+    }
+    else { toxmlp($q,\%v, join("",@{$c}))}
+  }
+  elsif (ref($c) eq "HASH") {
+    "<$q".
+      join("",map {" $_=\"$v{$_}\""} keys %v ) . ">" .
+	join("",map {($_ ne "-pcdata")
+		       ? ( (ref($c->{$_}) eq "ARRAY")
+			   ? "<$_>".
+			   join("</$_>\n<$_>", @{$c->{$_}}).
+			   "</$_>\n"
+			   : "<$_>$c->{$_}</$_>\n" )
+			 : () }
+	     keys %{$c} ) .
+	       "$c->{-pcdata}</$q>" }
 }
 
 sub mkdtskel{
- my $file = shift;
- my %mkdtskel=( '-default' => sub{$element{$q}++; 
-                              for (keys %v){$att{$q}{$_}=1 }; ""},
+  my @files = @_;
+  my %mkdtskel = (
+		  '-default' => sub{$element{$q}++; 
+				    for (keys %v){$att{$q}{$_}=1 }; ""},
 
-             '-end' => sub{ print <<'END';
+		  '-end' => sub{ print <<'END';
 #!/usr/bin/perl
 use XML::DT ;
 my $filename = shift;
@@ -704,27 +860,28 @@ my $filename = shift;
 #    '-outputenc' => 'ISO-8859-1',
 #    '-default'   => sub{"<$q>$c</$q>"},
 END
-
-for $name (keys %element){
-  print "     '$name' => sub{\"\$q:\$c\"},";
-  print '# remember $v{', join('},$v{',keys %{$att{$name}}), '}' if $att{$name};
-  print "# $element{$name}";
-  print "\n";
-}
-
-print <<'END';
+				 for $name (keys %element) {
+				   print "     '$name' => sub{\"\$q:\$c\"},";
+				   print '# remember $v{', join('},$v{',keys %{$att{$name}}), '}'
+				     if $att{$name};
+				   print "# $element{$name}";
+				   print "\n";
+				 }
+				 print <<'END';
 );
-print dt($filename,%handler); 
+print dt($filename,%handler);
 END
-            }
- );
+			       }
+		 );
 
- if($file eq "-latin1"){
-     $mkdtskel{'-inputenc'} = 'ISO-8859-1';
-     $file = shift;
- }
+  $file=shift(@files);
+  while($file =~ /^-/){
+    if   ($file eq "-html")   { $mkdtskel{'-html'} = 1;} 
+    elsif($file eq "-latin1") { $mkdtskel{'-inputenc'}='ISO-8859-1';}
+    else { die("usage mktskel [-html] [-latin1] file \n")}
+    $file=shift(@files)}
 
- dt($file,%mkdtskel)
+  dt($file,%mkdtskel)
 }
 
 sub nodeAttributes {
@@ -733,7 +890,9 @@ sub nodeAttributes {
   my @attrs = $node->getAttributes();
   for (@attrs) {
     if (ref($_) eq "XML::LibXML::Namespace") {
-      # TODO: This should not be ignored, I thing
+      # TODO: This should not be ignored, I think.
+      # This sould be converted on a standard attribute with
+      # key 'namespace' and respective contents
     } else {
       $answer{$_->getName()} = $_->getValue();
     }
@@ -746,23 +905,25 @@ sub mkdtdskel {
   my %handler=(
     '-outputenc' => 'ISO-8859-1',
     '-default'   => sub{ 
-          $elel{$q}=1;
+          $elel{$q}++;
           $root = $q unless ctxt(1);
-          $ele{ctxt(1)}{$q} = 1;
-          for(keys(%v)){$att{$q}{$_} = 1 } ;
+          $ele{ctxt(1)}{$q} ++;
+          for(keys(%v)){$att{$q}{$_} ++ } ;
         },
     '-pcdata'    => sub{ if ($c =~ /[^ \t\n]/){ $ele{ctxt(1)}{"#PCDATA"}=1 }},
   );
-  if($files[0] eq "-latin1"){
-     $handler{'-inputenc'} = 'ISO-8859-1';
-     shift @files;
-  }
+
+  while($files[0] =~ /^-/){
+    if   ($files[0] eq "-html")   { $handler{'-html'} = 1;} 
+    elsif($files[0] eq "-latin1") { $handler{'-inputenc'}='ISO-8859-1';}
+    else { die("usage mkdtdskel [-html] [-latin1] file* \n")}
+    shift(@files)}
 
   for $filename (@files){
   dt($filename,%handler); 
   }
 
-  print "<!-- DTD $root ... -->\n<!-- (C) ... " . `date`." -->\n";
+  print "<!-- DTD $root ... -->\n<!-- (C) ... " . localtime(time) ." -->\n";
   delete $elel{$root};
 
   for ($root, keys %elel){
@@ -773,7 +934,7 @@ sub mkdtdskel {
     }
   }
 }
-  
+
 
 sub process_libxml {
   my $node = shift;
@@ -792,23 +953,31 @@ sub process_libxml {
 
 
 sub putele {
-  my $e = shift; 
+  my $e = shift;
   my @f ;
-  if($ele{$e}){
-     @f = keys %{$ele{$e}};
-     print "<!ELEMENT $e (", join("|", @f ),")", 
-           (@f == 1 && $f[0] eq "#PCDATA" ? "" : "*"),
-           " >\n";
+  if ($ele{$e}) {
+    @f = keys %{$ele{$e}};
+    print "<!ELEMENT $e (", join("|", @f ),")",
+      (@f >= 1 && $f[0] eq "#PCDATA" ? "" : "*"),
+	" >\n";
+    print "<!-- ", join(" | ", (map {"$_=$ele{$e}{$_}"} @f )), " -->\n";
   }
   else {
-     print "<!ELEMENT $e  EMPTY >\n";}
+    print "<!ELEMENT $e  EMPTY >\n";
+  }
 }
 
+sub whitepc {
+  $_[1] eq '-pcdata' and $_[0] =~ /^[ \t\n]*$/
+}
 
-sub whitepc{ $_[1] eq '-pcdata' and $_[0] =~ /^[ \t\n]*$/ };
+sub MMAPON {
+  bless([@_],"mmapon")
+}
 
-sub MMAPON{ bless([@_],"mmapon") };
-sub SEQOF{ bless([@_],"seqof") };
+sub SEQOF {
+  bless([@_],"seqof")
+}
 
 package lat1;
 
